#define ORBITER_MODULE

#include "Luna9ALS.h"
#include "time.h"
#include "stdio.h"
#include "windows.h"





Luna9ALS::Luna9ALS (OBJHANDLE hObj, int fmodel): VESSEL2M (hObj, fmodel)
{
	// mesh
	VECTOR3 MeshOffset = _V(0, 0, 0);
	ClearMeshes();
	int MeshID = AddMesh ("Luna9ALS", &MeshOffset);
	SetMeshVisibilityMode (MeshID, MESHVIS_ALWAYS);

	// attachment points to KTDU (parent) and two airbags (child)
	CreateAttachment( TOPARENT, _V(0,0,0), _V(0,0,-1), _V(1,0,0), "ForKTDU", NOTLOOSE );
	hForAb1 = CreateAttachment( TOCHILD, _V(0,0,0.095), _V(0,0,1), _V( cos(45*RAD),-cos(45*RAD), 0), "ForAb1", NOTLOOSE );
	hForAb2 = CreateAttachment( TOCHILD, _V(0,0,0.095), _V(0,0,1), _V(-cos(45*RAD), cos(45*RAD), 0), "ForAb2", NOTLOOSE );
	AbMasses = 0;

	// define animations and animation defaults
	bAnimate = false;
	bBagsInflated = false;
	PetalsStatus = CLOSED;
	AntennasStatus = CLOSED;
	AnimationPercent = 0;
	CurrentAnimSpeed = ANIM_SPEED_PETAL;
	CurrentAnimType = PETALS;
	PetalsStatus = AntennasStatus = CurrentAnimStatus = CLOSED;
	PetalsAnimation = AntennasAnimation = CurrentAnimation = NULL;
	DefineAnimations();

	// photo panel
	bBtnStartActive = true;
	bPhotoSession = false;
	bMoveCamera = false;
	FrameStart = 0;
	PhotoPercent = 0;
	PercentedPhoto = 0;
	CameraDir = _V(0,0,0);
	CameraPosDeg = 0;

	int i;
	for (i=0; i<12; i++)
		hBmpPhotos[i] = NULL;

	srfBtnS = 0;

	for (i=0; i<2; i++)
	{
		srfBtnLR[i] = 0;
		srfBtnThmb[i] = 0;
	}

	for (i=0; i<3; i++)
	{
		srfPhotos[i] = 0;
		srfBtnThmb[i] = 0;
	}

	// timer
	Timer = 0;

	// what type of crater are we creating?
	WreckMeshID = MSH_WRECK_ALS_V2M;
}


Luna9ALS::~Luna9ALS ()
{
	for (int i =0; i<12; i++)
		if (hBmpPhotos[i] != NULL)
			delete hBmpPhotos[i];

	ReleaseSurfaces();
	ReleasePhotoSurfaces();
}


// General initialization of general params, called before config file
void Luna9ALS::clbkSetClassCaps (FILEHANDLE cfg)
{
	// Set geometry
	SetSize (1);

	// Set physics 
	SetLiftCoeffFunc (0);

	SetPMI (_V(3, 3, 3));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetRotDrag (_V(1.5,1.5,1.5));
	SetCW (0.2, 0.2, 0.2, 0.2);

		// surface
	SetSurfaceFrictionCoeff (10.0, 10.0);
	SetCOG_elev (1);
}	



// Read status from scenario file
void Luna9ALS::ParseScenarioLineEx(char *line, void *status)
{
	int tempint;

	// Airbag 1 name
	if (!strnicmp (line, "AB1", 3))
		sscanf (line+3, "%s", &Ab1Name);

	// Airbag 2 name
	else if (!strnicmp (line, "AB2", 3))
		sscanf (line+3, "%s", &Ab2Name);

	// petals state or incomplete animation
	else if (!strnicmp (line, "PETALS_STATUS", 13))
	{
		sscanf (line+13, "%lf", &AnimationPercent);
			
		if (AnimationPercent == 1)
			PetalsStatus = OPENED;
		else 
			CurrentAnimType = PETALS;
	}

	// antennas state or incomplete animation
	else if (!strnicmp (line, "ANTENNAS_STATUS", 15))
	{
		sscanf (line+15, "%lf", &AnimationPercent);

		if (AnimationPercent == 1)
			AntennasStatus = OPENED;
		else 
			CurrentAnimType = ANTENNAS;
	}

	// airbags state
	else if (!strnicmp (line, "BAGS_STATUS", 11))
		sscanf (line+11, "%d", &bBagsInflated);

	// timer
	else if (!strnicmp (line, "TIMER", 5))
		sscanf (line+5, "%lf", &Timer);


	// photo panel
	else if (!strnicmp (line, "PHOTO_STATE", 11))
	{
		sscanf (line+11, "%d", &tempint);

		if (tempint == 1)
			bPhotoSession = true;

		if (tempint > 0)
			bBtnStartActive = false;
	}

	else if (!strnicmp (line, "FRAME", 5))
		sscanf (line+5, "%d", &FrameStart);

	else if (!strnicmp (line, "SCAN_PHOTO", 10))
		sscanf (line+10, "%d", &PercentedPhoto);

	else if (!strnicmp (line, "SCAN_PERCENT", 12))
		sscanf (line+12, "%lf", &PhotoPercent);

	else if (!strnicmp (line, "CAMERA_DIR", 10))
		sscanf (line+10, "%lf", &CameraPosDeg);


	// VESSEL2M and VESSEL2 stuff
	else 
		VESSEL2M::ParseScenarioLineEx(line, status);
}


// Save status to scenario file
void Luna9ALS::clbkSaveState (FILEHANDLE scn)
{
	// do inherited VESSEL2M stuff before writing custom stuff
	VESSEL2M::clbkSaveState (scn);

	// airbags names
	if (Ab1Name[0] != NULL)
		oapiWriteScenario_string (scn, "AB1", Ab1Name);
	if (Ab2Name[0] != NULL)
		oapiWriteScenario_string (scn, "AB2", Ab2Name);

	// animating right now!
	if (bAnimate)
	{
		if (CurrentAnimation == PetalsAnimation)
			oapiWriteScenario_float (scn, "PETALS_STATUS", AnimationPercent);
		else
			oapiWriteScenario_float (scn, "ANTENNAS_STATUS", AnimationPercent);
	}
	
	// ... or not animating right now - write only if fully opened
	if (PetalsStatus == OPENED)
		oapiWriteScenario_int (scn, "PETALS_STATUS", PetalsStatus);
	if (AntennasStatus == OPENED)
		oapiWriteScenario_int (scn, "ANTENNAS_STATUS", AntennasStatus);

	// if we return, we'll have to know if the bags are inflated (only if still attached).
	// no way to get it from the bags themselves at this point.
	if ( (bBagsInflated) && (GetAttachmentStatus(hForAb1)) )
		oapiWriteScenario_int (scn, "BAGS_STATUS", bBagsInflated);

	// Timer
	if (Timer > 0)
		oapiWriteScenario_float (scn, "TIMER", Timer);


	// photo panel
	if (!bBtnStartActive)
	{
		if (bPhotoSession)
			oapiWriteScenario_int (scn, "PHOTO_STATE", 1);
		else
			oapiWriteScenario_int (scn, "PHOTO_STATE", 2);
	}

	if (FrameStart > 0)
		oapiWriteScenario_int (scn, "FRAME", FrameStart);

	if (PercentedPhoto > 0)
		oapiWriteScenario_int (scn, "SCAN_PHOTO", PercentedPhoto);

	if (PhotoPercent > 0)
		oapiWriteScenario_float (scn, "SCAN_PERCENT", PhotoPercent);

	if (CameraPosDeg > 0)
		oapiWriteScenario_float (scn, "CAMERA_DIR", CameraPosDeg);
}


// called after all vessel in scn are created, but before first timestep
void Luna9ALS::	FinishConfiguration()
{
	// airbags
	SetupAirbags();

	// configuration
	if (PetalsStatus == OPENED)
		FastForwardAnimation(PETALS);

	if (AntennasStatus == OPENED)
		FastForwardAnimation(ANTENNAS);

	// unfinished animation, just like the new one, we only pteempted the AnimationPercent value
	if ( (AnimationPercent > 0) && (AnimationPercent < 1) )
		AnimationStart(CurrentAnimType);

	// physics
	UpdatePhysics();

	// VESSEL2M stuff
	VESSEL2M::FinishConfiguration();
}




int Luna9ALS::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

	// only process keydown events
	if (!down) 
		return 0; 

	// shift combinations are reserved
	if (KEYMOD_SHIFT (kstate)) 
		return 0; 

	// control combinations are reserved
	if (KEYMOD_CONTROL (kstate))
		return 0; 

	if (key == OAPI_KEY_J) 
		SeparateAirbags();

	if (key == OAPI_KEY_K) 
		AnimationStart(PETALS);

	if (key == OAPI_KEY_G) 
		AnimationStart(ANTENNAS);

	// manual debug
	if (key == OAPI_KEY_D) 
		DebugSomething();

	return 0;
}


void Luna9ALS::clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel)
{
	// if we are loosing focus, clear the output
	if (hOldVessel == GetHandle())
		sprintf(oapiDebugString(), "");
}


void Luna9ALS::clbkPostStep(double simt, double simdt, double mjd)
{
	Simdt = simdt;

	// animation
	if (bAnimate)
		Animate();

	// timer
	if (Timer > 0)
		UpdateTimer();

	// photo session
	if (bPhotoSession)
		PhotoSession();

	if (bMoveCamera)
		MoveCamera();

	// common processing for all VESSEL2-derived objects
	TimestepForVESSEL2M();
}



void Luna9ALS::UpdateTimer()
{
	// update timer
	Timer += Simdt;

	// report timer 
	if (oapiGetFocusObject() == GetHandle())
	{
		if (Timer < T_AIRBAGS)
			sprintf(oapiDebugString(), "Airbags separation in %.0f s", T_AIRBAGS-Timer);
		else if (Timer < T_PETALS)
			sprintf(oapiDebugString(), "Petals open in %.0f s", T_PETALS-Timer);
		else if (Timer < T_ANTENNAS)
			sprintf(oapiDebugString(), "Antennas open in %.0f s", T_ANTENNAS-Timer);
		else if (Timer < 600)
			sprintf(oapiDebugString(), "Timer stops in %.0f s", T_TIMER-Timer);
	}

	// timer sound
	PlayVesselWave3(SoundLibID, SND_TIMER, LOOP);

	// pyro sound
	if (Timer < T_PYRO)
		PlayVesselWave3(SoundLibID, SND_PYRO);

	// separate airbags
	if ((Timer > T_AIRBAGS) && (GetAttachmentStatus(hForAb1)))
		SeparateAirbags();

	// open petals
	else if ((Timer > T_PETALS) && (PetalsStatus == CLOSED))
		AnimationStart(PETALS);

	// open antennas
	else if ((Timer > T_ANTENNAS) && (AntennasStatus == CLOSED))
		AnimationStart(ANTENNAS);

	// stop timer
	else if (Timer > T_TIMER)
	{
		Timer = 0;
		StopVesselWave3(SoundLibID, SND_TIMER);

		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(), "Timer stopped. Proceed to photo session! Open photo panel in the internal view by F8 key.");
	}
}


void Luna9ALS::UpdatePhysics(bool Animation)
{
	// airbags attached?
	if 	(Ab1Name[0] != NULL)
	{
		AbMasses = AB_MASS * 2;

		// airbags inflated
		if (bBagsInflated)
		{
			VvCrash = 40; 
			VhCrash = 40;
			VMaxBlast = 1000; 
			QMax = DRand(50000, 5000);

			SetCrossSections (_V(1.5, 1.5, 1.5));
			SetTouchdownPoints (_V( 1, 1,-Z_INFLATED), _V( 1, -1,-Z_INFLATED), _V(-1, 0,-Z_INFLATED));
		}

		else // not inflated
		{
			VvCrash = 20; 
			VhCrash = 20;
			VMaxBlast = 1000; 
			QMax = DRand(60000, 5000);

			SetCrossSections (_V(1.2, 1.2, 1.2));
			SetTouchdownPoints (_V( 1, 1,-Z_DEFLATED), _V( 1, -1,-Z_DEFLATED), _V(-1, 0,-Z_DEFLATED));
		}
	}

	else  // no airbags
	{
		AbMasses = 0;

		// don't do it after petal opening, or ALS jumps and turns.
		if (!Animation)
			SetTouchdownPoints (_V( 0, 1, -0.025), _V( 1, 0, -0.025), _V(0, 0, -0.325)); // inclined

		// petals closed
		if (PetalsStatus == CLOSED)
		{
			VvCrash = 10; 
			VhCrash = 10;
			VMaxBlast = 1000; 
			QMax = DRand(70000, 5000);

			SetCrossSections (_V(1.1, 1.1, 1.0));
		}

		else // petals open
		{
			VvCrash = 3; 
			VhCrash = 3;
			VMaxBlast = 1000; 
			QMax = DRand(20000, 5000);

			SetCrossSections (_V(1.2, 1.2, 3.0));
		}
	}

	SetEmptyMass(ALS_MASS + AbMasses);
}



void Luna9ALS::SetupAirbags()
{
	// default to non-attached
	AbMasses = 0;

	// should we attach? 
	// Do NOT allow attachment if only one airbag is present - this means a messed-up scn file!
 	if  ((Ab1Name[0] == 0) || (Ab2Name[0] == 0) )
		return;

	// do the objects exist?
	OBJHANDLE hAb1 = oapiGetObjectByName(Ab1Name);
	OBJHANDLE hAb2 = oapiGetObjectByName(Ab2Name);
	if ((!hAb1) || (!hAb2))
		return;

	// get child vessel interface
	VESSEL	*Intf1, *Intf2;
	Intf1 = oapiGetVesselInterface(hAb1);
	Intf2 = oapiGetVesselInterface(hAb2);

	// do children have proper attachment points? 
	if ((Intf1->AttachmentCount(TOPARENT) == 0) || (Intf2->AttachmentCount(TOPARENT) == 0) )
		return;

	// get attachment handles from children
	OBJHANDLE hToMe1 = Intf1->GetAttachmentHandle(TOPARENT, 0);
	OBJHANDLE hToMe2 = Intf2->GetAttachmentHandle(TOPARENT, 0);

	// Attach!
	AttachChild(hAb1, hForAb1, hToMe1);
	AttachChild(hAb2, hForAb2, hToMe2);
}


void Luna9ALS::InflateAirbags()
{
	// do we have someting to inflate? also, do it simultanously
    if ( (!GetAttachmentStatus(hForAb1)) || (!GetAttachmentStatus(hForAb2)) )
		return;

	PlayVesselWave3(SoundLibID, SND_INFLATE);

	SendVesselMessage(Ab1Name, VM_INFLATE);
	SendVesselMessage(Ab2Name, VM_INFLATE);

	bBagsInflated = true;
	UpdatePhysics();
}


// by J key, by autopilot or by accident :-)
void Luna9ALS::SeparateAirbags()
{
	// do we have someting to separate? also, do it simultanously
    if ( (!GetAttachmentStatus(hForAb1)) || (!GetAttachmentStatus(hForAb2)) )
		return;

	PlayVesselWave3(SoundLibID, SND_PYRO);

	// Better to randomize the kick effect
	double vDetach = 0.1; 

	// separate whatever is attached to the payload attacment, indiscriminatory
	DetachChild(hForAb1, vDetach);
	SendVesselMessage(Ab1Name, VM_DETACH);
	DetachChild(hForAb2, vDetach);
	SendVesselMessage(Ab2Name, VM_DETACH);

	// do the last kick 
	OBJHANDLE hAb1 = oapiGetObjectByName(Ab1Name);
	OBJHANDLE hAb2 = oapiGetObjectByName(Ab2Name);

	// get child vessel interface
	VESSEL	*Intf1, *Intf2;
	Intf1 = oapiGetVesselInterface(hAb1);
	Intf2 = oapiGetVesselInterface(hAb2);
	Intf1->AddForce( _V(3000, 0, 500), _V(0, 0, 0) );
	Intf2->AddForce( _V(3000, 0, 500), _V(0, 0, 0) );

	// kill AB names so it won't be stored in scn file (if it will, it can reactivate the attachments)
	Ab1Name[0] = 0;
	Ab2Name[0] = 0;

	bLandedPermanently = false;

	//
	UpdatePhysics();
}


void Luna9ALS::GetVesselMessage(VesselMessage VM)
{
	if (VM == VM_SEPARATE_CHILD)
		SeparateAirbags();

	else if (VM == VM_INFLATE)
		InflateAirbags();

	else if (VM == VM_TIMER)
	{
		Timer = 0.01;
		bFocus = true;
	}

	// VESSEL2M stuff
	else
		VESSEL2M::GetVesselMessage(VM);
}



// Enforce camera dir switch here:
bool Luna9ALS::clbkLoadGenericCockpit ()
{
	oapiSetDefRCSDisplay(0);
	SetCameraOffset(_V(0, 0, 0.45));
	SetCameraDefaultDirection(_V(cos(45*RAD)*cos(15*RAD), cos(45*RAD)*cos(15*RAD), -cos(75*RAD)), 75*RAD);
	return true;
}


	// ===== ANIMATIONS =====


void Luna9ALS::DefineAnimations()
{
	// Petals

	MGROUP_ROTATE *Petals[4];

	static UINT grPetal0[1] = {17}; 
	static UINT grPetal1[1] = {18}; 
	static UINT grPetal2[1] = {19}; 
	static UINT grPetal3[2] = {15,14}; 

	Petals[0] = new MGROUP_ROTATE(0, grPetal0, 1, _V( 0.0, 0.329, 0.0), _V(1, 0, 0), (float)(-120*PI/180));
	Petals[1] = new MGROUP_ROTATE(0, grPetal1, 1, _V(-0.329, 0.0, 0.0), _V(0, 1, 0), (float)(-120*PI/180));
	Petals[2] = new MGROUP_ROTATE(0, grPetal2, 1, _V(0.0, -0.329, 0.0), _V(1, 0, 0), (float)( 120*PI/180));
	Petals[3] = new MGROUP_ROTATE(0, grPetal3, 2, _V( 0.329, 0.0, 0.0), _V(0, 1, 0), (float)( 120*PI/180));

	PetalsAnimation = CreateAnimation (0);
	AddAnimationComponent (PetalsAnimation, 0, 1, Petals[0]);
	AddAnimationComponent (PetalsAnimation, 0, 1, Petals[1]);
	AddAnimationComponent (PetalsAnimation, 0, 1, Petals[2]);
	AddAnimationComponent (PetalsAnimation, 0, 1, Petals[3]);

	// Antennas

	MGROUP_ROTATE *Antennas[4];

	static UINT grAntenna0[1] = {16}; 
	static UINT grAntenna1[1] = {1}; 
	static UINT grAntenna2[1] = {9}; 
	static UINT grAntenna3[1] = {13}; 

	Antennas[0] = new MGROUP_ROTATE(0, grAntenna0, 1, _V(-0.217, 0.217, 0.135), _V(0.5,  0.5, 0), (float)(-80*PI/180));
	Antennas[1] = new MGROUP_ROTATE(0, grAntenna1, 1, _V(-0.217,-0.217, 0.135), _V(0.5, -0.5, 0), (float)( 80*PI/180));
	Antennas[2] = new MGROUP_ROTATE(0, grAntenna2, 1, _V( 0.217,-0.217, 0.135), _V(0.5,  0.5, 0), (float)( 80*PI/180));
	Antennas[3] = new MGROUP_ROTATE(0, grAntenna3, 1, _V( 0.217, 0.217, 0.135), _V(0.5, -0.5, 0), (float)(-80*PI/180));

	AntennasAnimation = CreateAnimation (0);
	AddAnimationComponent (AntennasAnimation, 0, 1, Antennas[0]);
	AddAnimationComponent (AntennasAnimation, 0, 1, Antennas[1]);
	AddAnimationComponent (AntennasAnimation, 0, 1, Antennas[2]);
	AddAnimationComponent (AntennasAnimation, 0, 1, Antennas[3]);
}


void Luna9ALS::FastForwardAnimation(anim_type AnimType)
{
	if (AnimType == PETALS)
		SetAnimation (PetalsAnimation, 1);
	else
		SetAnimation (AntennasAnimation, 1);

	AnimationEnd(AnimType, OPENED);
}


void Luna9ALS::Animate()
{
	double dPercent = CurrentAnimSpeed * Simdt;

	if (CurrentAnimType == PETALS)
		PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP);

	if (AnimationPercent < 1.0)
		AnimationPercent = min (1.0, AnimationPercent + dPercent);
	else
		AnimationEnd(CurrentAnimType, OPENED);

	SetAnimation (CurrentAnimation, AnimationPercent);
}


void Luna9ALS::AnimationStart(anim_type AnimType)
{
	// already animating something? ignore new animation request until done.
	if (bAnimate)
		return;

	// don't animate if airbags are still there
    if ( (GetAttachmentStatus(hForAb1)) || (GetAttachmentStatus(hForAb2)) )
		return;

	// set up animation. these values depend on animation object...
	if (AnimType == PETALS)
	{
		if (PetalsStatus == OPENED)
			return;

		CurrentAnimStatus = PetalsStatus;
		CurrentAnimation = PetalsAnimation;
		CurrentAnimSpeed = ANIM_SPEED_PETAL;
	}
	else
	{
		if (AntennasStatus == OPENED)
			return;

		CurrentAnimStatus = AntennasStatus;
		CurrentAnimation = AntennasAnimation;
		CurrentAnimSpeed = ANIM_SPEED_ANTENNA;

		PlayVesselWave3(SoundLibID, SND_PYRO);
	}

	// ...these don't 
	CurrentAnimType = AnimType;

	// cross-condition for restricting dependences between animations
	if ( 
		 ( (AnimType == PETALS) && (AntennasStatus == OPENED) )
		 ||
		 ( (AnimType == ANTENNAS) && (PetalsStatus == CLOSED) )
	   )
	   return;

	// finally, allow animation
	bAnimate = true;
}


void Luna9ALS::AnimationEnd(anim_type AnimType, anim_status NewStatus)
{
	bAnimate = false;

	if (AnimType == PETALS)
	{
		PetalsStatus = NewStatus;
		UpdatePhysics(ANIMATION);
		StopVesselWave3(SoundLibID, SND_MOTOR);
	}
	else
	{
		AntennasStatus = NewStatus;
		PlayVesselWave3(SoundLibID, SND_CLANG);
	}
}


				// 2D PANEL


void Luna9ALS::ReleaseSurfaces ()
{
	int i;

	if (srfBtnS != NULL)
		oapiDestroySurface(srfBtnS);
	srfBtnS = 0;

	for (i=0; i<2; i++)
		if (srfBtnLR[i] != NULL)
		{
			oapiDestroySurface(srfBtnLR[i]);
			srfBtnLR[i] = 0;
		}

	for (i=0; i<2; i++)
		if (srfBtnThmb[i] != NULL)
		{
			oapiDestroySurface(srfBtnThmb[i]);
			srfBtnThmb[i] = 0;
		}
}


void Luna9ALS::ReleasePhotoSurfaces()
{
	// reset surfaces to empty
	for (int i=0; i<3; i++)
		if (srfPhotos[i] != NULL)
		{
			oapiDestroySurface(srfPhotos[i]);
			srfPhotos[i] = 0;
		}
}


void Luna9ALS::ReloadPhotoSurfaces()
{
	// reset to empty
	ReleasePhotoSurfaces();

	// reload new pictures
	for (int i=0; i<3; i++)
		srfPhotos[i] = oapiCreateSurface (hBmpPhotos[FrameStart+i], false);
}


void Luna9ALS::RedrawPhotoSurfaces()
{
	int i;

	// completely discard old and reload new pictures in memory
	ReloadPhotoSurfaces();

	// redraw navigation buttons
	for (i=0; i<2; i++)
		oapiTriggerPanelRedrawArea (PanelId, AID_BTN_LEFT+i);

	// redraw thumbnails
	for (i=0; i<12; i++)
		oapiTriggerPanelRedrawArea (PanelId, AID_THMB_01+i);

	// redraw new pictures
	for (i=0; i<3; i++)
		oapiTriggerPanelRedrawArea (PanelId, AID_PHOTO_1+i);
}


// photo width: full, empty or in between. aid is the AID_
int Luna9ALS::PhotoWidth(int aid)
{
	// scan did not start, empty
	if (bBtnStartActive)
		return 0;

	// most complex, incomplete scan
	else if (bPhotoSession)
	{
		int PhotoId = aid - AID_PHOTO_1 + FrameStart;

		if (PhotoId < PercentedPhoto)	// left of incomplete, full draw
			return 345;
		if (PhotoId > PercentedPhoto)	// right of incomplete, empty draw
			return 0;
		else							// incomplete draw
			return int(345 * PhotoPercent);
	}

	// scaning is complete, all pictures are full
	else 
		return 345;
}


int Luna9ALS::ThumbnailWidth(int aid)
{
	// scan did not start, empty
	if (bBtnStartActive)
		return 0;

	// most complex, incomplete scan
	else if (bPhotoSession)
	{
		int ThumbId = aid - AID_THMB_01;

		if (ThumbId < PercentedPhoto)	// left of incomplete, full draw
			return 25;
		if (ThumbId > PercentedPhoto)	// right of incomplete, empty draw
			return 0;
		else							// incomplete draw
			return int(25 * PhotoPercent);
	}

	// scaning is complete, all thumbnails are full
	else 
		return 25;
}


int Luna9ALS::ThumbnailColor(int aid)
{
	int ThumbId = aid - AID_THMB_01;

	// scan did not start, does not really matter, paint width will be 0
	if (bBtnStartActive)
		return 0;

	// incomplete scan
	else if (bPhotoSession)
	{
		// left of or incomplete? pale!
		if (ThumbId <= PercentedPhoto)	
			return 1;
		else
			return 0; // does not really matter, paint width will be 0
	}


	// most complex, complete scan, navigation
	else
	{
		// within displayed frame?
		if ( (ThumbId >= FrameStart) && (ThumbId <= (FrameStart+2)) )
			return 2;
		else
			return 0;
	}
}


bool Luna9ALS::clbkLoadPanel (int id)
{
	int i;
	HBITMAP hBmp;
	int xShift;

	// allow to load panel only if we are landed and antenans are opened
	if ( (!bLandedPermanently) || (AntennasStatus != OPENED) )
		return false;

	// load panel background (try with two alternative panels)
	if (id == 0)
	{
		hBmp = LOADBMP ("Textures\\Luna9\\Panel1.bmp");
		if (hBmp == NULL)
			return false;

		xShift = 0;
		CameraSpeed = CAMERA_MOVE_SPEED_DEG;
		oapiSetPanelNeighbours (-1, -1, -1, 1); 
		PanelId = 0;
	}
	else if (id == 1)
	{
		hBmp = LOADBMP ("Textures\\Luna9\\Panel2.bmp");
		if (hBmp == NULL)
			return false;

		xShift = 441;
		CameraSpeed = CAMERA_JUMP_SPEED_DEG;
		oapiSetPanelNeighbours (-1, -1, 0, -1); 
		PanelId = 1;
	}
	else
		return false;

	// load photo bitmaps only once
	if (hBmpPhotos[0] == NULL)
	{
		hBmpPhotos[0] = LOADBMP ("Textures\\Luna9\\Pan01.bmp");
		hBmpPhotos[1] = LOADBMP ("Textures\\Luna9\\Pan02.bmp");
		hBmpPhotos[2] = LOADBMP ("Textures\\Luna9\\Pan03.bmp");
		hBmpPhotos[3] = LOADBMP ("Textures\\Luna9\\Pan04.bmp");
		hBmpPhotos[4] = LOADBMP ("Textures\\Luna9\\Pan05.bmp");
		hBmpPhotos[5] = LOADBMP ("Textures\\Luna9\\Pan06.bmp");
		hBmpPhotos[6] = LOADBMP ("Textures\\Luna9\\Pan07.bmp");
		hBmpPhotos[7] = LOADBMP ("Textures\\Luna9\\Pan08.bmp");
		hBmpPhotos[8] = LOADBMP ("Textures\\Luna9\\Pan09.bmp");
		hBmpPhotos[9] = LOADBMP ("Textures\\Luna9\\Pan10.bmp");
		hBmpPhotos[10] = LOADBMP ("Textures\\Luna9\\Pan11.bmp");
		hBmpPhotos[11] = LOADBMP ("Textures\\Luna9\\Pan12.bmp");
	}

	for (i =0; i<12; i++)
		if (hBmpPhotos[i] == NULL)
			return false;

	// cleanup from any previous load panel
	ReleaseSurfaces();

	oapiRegisterPanelBackground (hBmp, PANEL_ATTACH_TOP, 0xffffff);

	// start button
	srfBtnS = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\BtnS.bmp"));
	oapiRegisterPanelArea (AID_BTN_START, _R(xShift+107,  411, xShift+225, 445), PANEL_REDRAW_MOUSE, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);

	// left-right buttons
	srfBtnLR[0] = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\BtnL.bmp"));
	srfBtnLR[1] = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\BtnR.bmp"));
	oapiRegisterPanelArea (AID_BTN_LEFT,  _R(xShift+317,  414, xShift+342, 440), PANEL_REDRAW_USER, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);
	oapiRegisterPanelArea (AID_BTN_RIGHT, _R(xShift+697,  414, xShift+722, 440), PANEL_REDRAW_USER, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);

	// frame thumbnails
	srfBtnThmb[0] = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\ThmbBl.bmp"));
	srfBtnThmb[1] = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\ThmbPl.bmp"));
	srfBtnThmb[2] = oapiCreateSurface (LOADBMP ("Textures\\Luna9\\ThmbGr.bmp"));

	// navigation via thumbnails
	for (i =0; i<12; i++)
	  oapiRegisterPanelArea (AID_THMB_01+i,  _R(xShift+353+i*28,  413, xShift+379+i*28, 442), PANEL_REDRAW_USER, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);

	// photo pan windows
	oapiRegisterPanelArea (AID_PHOTO_1, _R(xShift+1,    1,  xShift+346, 401), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	oapiRegisterPanelArea (AID_PHOTO_2, _R(xShift+347,  1,  xShift+692, 401), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	oapiRegisterPanelArea (AID_PHOTO_3, _R(xShift+693,  1, xShift+1038, 401), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);

	// visuals
	sprintf(oapiDebugString(), "");
	SetCameraOffset(_V(0, 0, 0.35));
	ReloadPhotoSurfaces();
	JumpCamera();

	// OK?
	return hBmp != NULL;
}


bool Luna9ALS::clbkPanelMouseEvent (int id, int event, int mx, int my)
{
	// start photosession in time step
	if ( (id == AID_BTN_START) && (bBtnStartActive) )
	{
		sprintf(oapiDebugString(), "");
		bPhotoSession = true;
		bBtnStartActive = false;
		PastStepPercentedPhoto = 0;

		PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP, 200);

		return true;
	}

	// navigation buttons for fully scanned mode
	if ( (!bBtnStartActive) && (!bPhotoSession) )
	{
		// store current frame start
		double OldFrameStart = FrameStart;

		// update frame start, if needed

		if (id == AID_BTN_LEFT)
			FrameStart--;

		if (id == AID_BTN_RIGHT)
			FrameStart++;

		if (id >= AID_THMB_01) 
			FrameStart = id - AID_THMB_01 - 1;
	
		// enforce limits
		FrameStart = max(0, FrameStart);
		FrameStart = min(9, FrameStart);

		// trigger camera move, if needed
		if (OldFrameStart != FrameStart)
		{
			bMoveCamera = true;
			PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP, 180);
		}

		return true;
	}

	return false;
}

bool Luna9ALS::clbkPanelRedrawEvent (int id, int event, SURFHANDLE surf)
{
	// paint start button grey
	if ( (id == AID_BTN_START) && (!bBtnStartActive) )
	{
		oapiBlt (surf, srfBtnS, 0, 0, 0, 0, 118, 34);
		return true;
	}

	// paint one photo panel
	if ( (id >= AID_PHOTO_1) && (id <= AID_PHOTO_3) )
	{
		oapiBlt (surf, srfPhotos[id - AID_PHOTO_1], 0, 0, 0, 0, PhotoWidth(id), 400);
		return true;
	}

	// navigation buttons for fully scanned mode
	if ( (!bBtnStartActive) && (!bPhotoSession) )
	{
		// paint left button black
		if (id == AID_BTN_LEFT) 
		{
			if (FrameStart > 0)
				oapiBlt (surf, srfBtnLR[0], 0, 0, 0, 0, 25, 26);

			return true;
		}

		// paint right button black
		if (id == AID_BTN_RIGHT) 
		{
			if (FrameStart < 9) 
				oapiBlt (surf, srfBtnLR[1], 0, 0, 0, 0, 25, 26);

			return true;
		}
	}

		// paint thumbnail black, pale or green
	if ( !bBtnStartActive)
	{
		if ( (id >= AID_THMB_01) && (id <= AID_THMB_12) )
		{
			oapiBlt (surf, srfBtnThmb[ThumbnailColor(id)], 0, 0, 0, 0, ThumbnailWidth(id), 28);
			return true;
		}
	}

	return false;
}



void Luna9ALS::JumpCamera()
{
	// target X angle, degrees
	double AngleXdeg;

	if (bBtnStartActive)							// pre-scan: go to beginning 
		AngleXdeg = PAN_START_DEG;
	else											// scan and post-scan: go to curent
		AngleXdeg = CameraPosDeg;

	// camera direction vector
	CameraDir.x = cos(AngleXdeg * RAD);
	CameraDir.y = sqrt(1-CameraDir.x*CameraDir.x);
	CameraDir.z = 0;

	// camera tilt
	double CameraTiltRad;

	if (AngleXdeg >= 90)
		CameraTiltRad = -90*RAD;
	else
		CameraTiltRad =  90*RAD;

	// move camera to new position
	SetCameraDefaultDirection(CameraDir, CameraTiltRad);
}


void Luna9ALS::PhotoSession()
{
	double dAngleXdeg = Simdt * CAMERA_SCAN_SPEED_DEG;
	CameraPosDeg += dAngleXdeg;

	PercentedPhoto = int(CameraPosDeg / PAN_SEG_DEG);

	// new photo? play photo sound for a previous frame
	if (PastStepPercentedPhoto != PercentedPhoto)
		PlayVesselWave3(SoundLibID, SND_PHOTO);
	PastStepPercentedPhoto = PercentedPhoto;

	double CurrPhotoStartAngleDeg = PAN_SEG_DEG * PercentedPhoto;
	double CurrPhotoAngleDeg = CameraPosDeg - CurrPhotoStartAngleDeg;
	PhotoPercent = CurrPhotoAngleDeg / PAN_SEG_DEG;

	FrameStart = max(0, PercentedPhoto-2);

	// debug
//	sprintf(oapiDebugString(), "X deg %f   CameraPosDeg %f, PercentedPhoto %d  PhotoPercent %f   FrameStart %d", 
//		acos(CameraDir.x)*DEG, CameraPosDeg, PercentedPhoto, PhotoPercent, FrameStart);

	// done?
	if (PercentedPhoto == 12)
	{
		bPhotoSession = false;
		bMoveCamera = true;
		FrameStart = 3;
		PercentedPhoto = 0;
		PhotoPercent = 0;

		PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP, 180);
		PlayVesselWave3(SoundLibID, SND_PHOTO);
	}
	else		// not yet!
	{
		// update visuals
		JumpCamera();
		RedrawPhotoSurfaces();
		PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP, 200);
	}
}


// this is guaranteed to be called only when full scan is completed
void Luna9ALS::MoveCamera()
{
	double dAngleXdeg = Simdt * CameraSpeed;

	// target angle
	double TargetAngXdeg = (FrameStart + 1) * PAN_SEG_DEG;

	// shift camera left or right
	if (CameraPosDeg > TargetAngXdeg)
		CameraPosDeg -= dAngleXdeg;
	else
		CameraPosDeg += dAngleXdeg;

	PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP, 180);

	// done?
	if (fabs(TargetAngXdeg - CameraPosDeg) < 0.5)
	{
		bMoveCamera = false;
		RedrawPhotoSurfaces();
		StopVesselWave3(SoundLibID, SND_MOTOR);
	}

	JumpCamera();
}


void Luna9ALS::Land()
{
	if (!bLandedPermanently)
		PlayVesselWave3(SoundLibID, SND_BOUNCE, 0, 200);
}


void Luna9ALS::DebugSomething()
{
//	SetCameraDefaultDirection(_V(0, 0, 1), 0);
//	sprintf(oapiDebugString(), "%f, %f", VvCrash, vvv);
}


		// SOUND
void Luna9ALS::clbkPostCreation (void)
{
	SoundLibID = ConnectToOrbiterSoundDLL3(GetHandle());

	SoundOptionOnOff3(SoundLibID, PLAYMAINTHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYATTITUDETHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYLANDINGANDGROUNDSOUND, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAIRSPEED, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAMBIANCEWHENLANDED, true);

	SoundOptionOnOff3(SoundLibID, PLAYCOUNTDOWNWHENTAKEOFF, false);
	SoundOptionOnOff3(SoundLibID, PLAYWHENATTITUDEMODECHANGE, false);
	SoundOptionOnOff3(SoundLibID, PLAYGPWS, false);
	SoundOptionOnOff3(SoundLibID, PLAYHOVERTHRUST, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKINGSOUND, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADARBIP, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKLANDCLEARANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINAIRCONDITIONING, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINRANDOMAMBIANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADIOATC, false);
	SoundOptionOnOff3(SoundLibID, DISPLAYTIMER, false);

	RequestLoadVesselWave3(SoundLibID, SND_PYRO, "Sound\\Luna9\\Pyro.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_INFLATE, "Sound\\Luna9\\Inflate.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_MOTOR, "Sound\\Luna9\\Probe.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_CLANG, "Sound\\Luna9\\Clang.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_BOUNCE, "Sound\\Vessel\\Crash.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_TIMER, "Sound\\Luna9\\Clock.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_PHOTO, "Sound\\Luna9\\Photo.wav", BOTHVIEW_FADED_FAR);
}


// ==============================================================
// API interface
// ==============================================================


DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
}

DLLCLBK void ExitModule (HINSTANCE hModule)
{
	;
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	srand((unsigned)time( NULL ));
	return new Luna9ALS (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) 
		delete (Luna9ALS*)vessel;
}



