#include "..\..\include\orbitersdk.h"
#include "..\..\..\OrbiterSoundSDK\include\OrbiterSoundSDK35.h"
#include "Vessel2M.h"

const double ALS_MASS = 80;
const double AB_MASS = 10;
const double Z_INFLATED = 0.35;
const double Z_DEFLATED = 0.30;

const bool ANIMATION = true;
const double ANIM_SPEED_PETAL = 0.1;
const double ANIM_SPEED_ANTENNA = 4;


const double T_PYRO = 0.2;
const double T_AIRBAGS = 30;
const double T_PETALS = 50;
const double T_ANTENNAS = 70;
const double T_TIMER = 80;

typedef struct {
	HINSTANCE hDLL;
	HFONT font[2];
	DWORD col[4];
	HBRUSH brush[4];
	HPEN pen[2];
} GDIParams;

GDIParams g_Param;

#define LOADBMP(id) ( (HBITMAP) LoadImage(NULL, id, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION) )

const int AID_BTN_START	= 0;
const int AID_BTN_LEFT	= 1;
const int AID_BTN_RIGHT	= 2;

const int AID_PHOTO_1	= 10;
const int AID_PHOTO_2	= 11;
const int AID_PHOTO_3	= 12;

const int AID_THMB_01 = 21;
const int AID_THMB_12 = 32;


const double PAN_SEG_DEG = 11;
const double PAN_START_DEG = 0;
const double CAMERA_SCAN_SPEED_DEG = 3; // dg/sec
const double CAMERA_MOVE_SPEED_DEG = 30; // dg/sec
const double CAMERA_JUMP_SPEED_DEG = 300; // dg/sec

// sound
const int SND_PYRO = 1;
const int SND_INFLATE = 2;
const int SND_MOTOR = 3;
const int SND_CLANG = 4;
const int SND_BOUNCE = 5;
const int SND_TIMER = 6;
const int SND_PHOTO = 7;


class Luna9ALS: public VESSEL2M {
public:
	// constructor
	Luna9ALS (OBJHANDLE hObj, int fmodel);
	~Luna9ALS ();
	void ReleaseSurfaces();

	// callbacks
	void clbkSetClassCaps(FILEHANDLE cfg);
	void clbkPostStep(double simt, double simdt, double mjd);
	bool clbkLoadGenericCockpit ();
	void clbkPostCreation (void);
    int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	bool clbkLoadPanel (int id);
	bool clbkPanelMouseEvent (int id, int event, int mx, int my);
	bool clbkPanelRedrawEvent (int id, int event, SURFHANDLE surf);

	// VESSEL2M overrides
	void clbkSaveState (FILEHANDLE scn);
	void ParseScenarioLineEx(char *line, void *status);
	void FinishConfiguration ();
    void GetVesselMessage(VesselMessage VM);
    void Land();

private:
	// airbags attachments
	ATTACHMENTHANDLE hForAb1, hForAb2;			//+--
	char Ab1Name[255], Ab2Name[255];			//-++
	double AbMasses;							//+--
	int bBagsInflated;							//+++
	void SetupAirbags();
	void InflateAirbags();
	void SeparateAirbags();

	// runtime
	double Simdt;								//---

	// configuration
	void UpdatePhysics(bool Animation = false);

	// timer
	double Timer;								//+++
	void UpdateTimer();

	// animations
	bool bAnimate;								//+--
	double AnimationPercent;					//+++
	double CurrentAnimSpeed;					//+--
	enum anim_type {PETALS, ANTENNAS} CurrentAnimType; //+--
    enum anim_status { CLOSED, OPENED } PetalsStatus, AntennasStatus, CurrentAnimStatus; //+++
	UINT PetalsAnimation, AntennasAnimation, CurrentAnimation; //+--
	void DefineAnimations();
	void Animate();
	void FastForwardAnimation(anim_type AnimType);
	void AnimationStart(anim_type AnimType);
	void AnimationEnd(anim_type AnimType, anim_status NewStatus);

		// panels

	bool bBtnStartActive;						//+++
	bool bPhotoSession;							//+++
	int FrameStart;								//+++
	double PhotoPercent;						//+++
	int PercentedPhoto;							//+++
	int PastStepPercentedPhoto;					//---
	int PanelId;								//---

	SURFHANDLE srfBtnS;							//+--
	SURFHANDLE srfPhotos[3];					//+--
	SURFHANDLE srfBtnLR[2];						//+--
	SURFHANDLE srfBtnThmb[3];					//+-- black, pale, green
	HBITMAP hBmpPhotos[12];						//+--

	void PhotoSession();
	void ReleasePhotoSurfaces();
	void ReloadPhotoSurfaces();
	void RedrawPhotoSurfaces();
	int PhotoWidth(int aid);
	int ThumbnailWidth(int aid);
	int ThumbnailColor(int aid);

		// panel camera
	bool bMoveCamera;							//+--
	VECTOR3 CameraDir;							//+--
	double CameraPosDeg;						//+++      X only
	double CameraSpeed;							//---

	void CalcCameraDirAndTilt();
	void JumpCamera();
	void MoveCamera();

	// sound
	int SoundLibID;

	// output
	void DebugSomething();
};
