#include "..\..\include\orbitersdk.h"
#include "..\..\..\OrbiterSoundSDK\include\OrbiterSoundSDK35.h"
#include "Vessel2M.h"


const double EMPTY_MASS = 330;
const double PO_MASS = 150;
const double ALS9_MASS = 100;
const double ALS10_MASS = 245;
const double FUEL_MASS = 870;

const double THRUST_MAIN = 44100; //N
const double THRUST_STEER = 333; 
const double THRUST_STEER_ROLL = 49; 
const double ISP_MAIN = 2813; //Ed

// default Orbiter exhaust
//const double MAIN_Z = -1.9;
//const double MAIN_EXH_LENGTH = 5.0;
//const double MAIN_EXH_WIDTH = 0.4;
// custom particles/texture exhaust
//from Thorton example
//const double MAIN_Z = -1.3;
//const double MAIN_EXH_LENGTH = 2.35;
const double MAIN_Z = -1.5;
const double MAIN_EXH_LENGTH = 4.00;
const double MAIN_EXH_WIDTH = MAIN_EXH_LENGTH / 4.196;

const double STEER_XY = 0.590;
// default Orbiter exhaust
//const double STEER_Z = -0.85;
//const double STEER_EXH_LENGTH = 1.0;
//const double STEER_EXH_WIDTH = 0.075;
// custom particles/texture exhaust
const double STEER_Z = -0.77;
const double STEER_EXH_LENGTH = 1.0;
const double STEER_EXH_WIDTH = STEER_EXH_LENGTH / 4.196;

const double STEER_ROLL_X = 0.63; // model slightly asymmetrical, second nozzle is 0.64
//const double STEER_ROLL_Y = 0.595;
const double STEER_ROLL_Z = -0.667;
//const double STEER_ROLL_EXH_LENGTH = 0.2;
//const double STEER_ROLL_EXH_WIDTH = 0.015;
// custom particles/texture exhaust
const double STEER_ROLL_Y = 0.598;
const double STEER_ROLL_EXH_LENGTH = 0.2;
const double STEER_ROLL_EXH_WIDTH = STEER_ROLL_EXH_LENGTH / 4.196;

// steering system is virtualized. The original system warks all the time KTDU is running;
// steering is done by decreezing the steering thrust istead of increasing it. 
// Orbiter does not handle that well.
const double VIRT_RCS_FUEL_MASS = 1;
const double ISP_VIRT_RCS = 400000;
const double THRUST_VIRT_RSC_ROLL = 40;
const double THRUST_VIRT_RSC = 300;

// for debug only, to be disabled when OK
const double VIRT_RCS_EXH_LENGTH = 0;//1;  
const double VIRT_RCS_EXH_WIDTH = 0;//0.1; 

// gas RCS is also virtualized, but for different reason:
// the real system is too week to overcome disturbances induced bt time acceleration
const double GAS_RCS_FUEL_MASS = 1;
const double GAS_RCS_ISP = 400000;
const double THRUST_GAS_RSC = 50;
const double GAS_RCS_EXH_LENGTH = 0.15;
const double GAS_RCS_EXH_WIDTH = 0.004;

const double GAS_ROLL_X = 0.421;			
const double GAS_ROLL_Y = 0.062 + 0.739;	
const double GAS_ROLL_Z = 0.438;			
const double GAS_YAW_X = 0.54;
const double GAS_YAW_Y = 0.18 + 0.739;
const double GAS_YAW_Z = 0.005;
const double GAS_YAW_Z_2 = -0.044;
const double GAS_PITCH_X = -0.003;
const double GAS_PITCH_Y = 0.329 + 0.739;
const double GAS_PITCH_Z = 0.393;
const double GAS_PITCH_Z_2 = 0.44;

const double MCC_9_PERI_MIN = 10000;		// 10 km
const double MCC_10_PERI_MIN = 2738000;		// 1000 km
const double MCC_10_PERI_MAX = 1748000;		// 1010 km

#define XPLUS  _V( 1,  0,  0)
#define XMINUS _V(-1,  0,  0)
#define YPLUS  _V( 0,  1,  0)
#define YMINUS _V( 0, -1,  0)
#define ZPLUS  _V( 0,  0,  1)
#define ZMINUS _V( 0,  0, -1)


const double ALT_MCC_NOMINAL = 50000000; //100 th. km.
const double ALT_MCC_MAX = 4000000; //100 th. km.
const double ALT_TURN = 400000; //400 km.

const double ALT_TIMEACCEL_1000 = 8000000; // 8000 km
const double ALT_TIMEACCEL_100  = 2000000; // 2000 km

// on-screen message lifetime
const double MSG_DISPLAY_TIME = 3;

const double MAX_TIME_ACCEL = 10000;

const bool IGNORE_FOCUS = true;



const int LANDER = 0;
const int SATELLITE = 1;

// sound
const int SND_PYRO = 1;
const int SND_INFLATE = 2;
const int SND_PROBE = 3;



class Luna9KTDU: public VESSEL2M {
public:

	// constructor
	Luna9KTDU (OBJHANDLE hObj, int fmodel);

	// callbacks
	void clbkSetClassCaps(FILEHANDLE cfg);
	bool clbkLoadGenericCockpit ();
	void clbkPostCreation (void);
	void clbkPostStep(double simt, double simdt, double mjd);
    int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey (char *keystate);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);

	// VESSEL2M overrides
	void clbkSaveState (FILEHANDLE scn);
	void ParseScenarioLineEx(char *line, void *status);
	void FinishConfiguration ();
	void Land();

private:

	// main engine 
	PROPELLANT_HANDLE MainTank;							//--- 
	int MainThGroupCount;								//+++
	THRUSTER_HANDLE MainEngine[7];						//---
	THGROUP_HANDLE MainThGroup;							//---
	void ToggleMainThrustChamber();
	void EnableMainThrustChamber();
	void DisableMainThrustChamber();
	
	// virtual RCS
	PROPELLANT_HANDLE VirtualRCSTank;					//---
	
	// PO RCS, gas
	PROPELLANT_HANDLE GasRCSTank;						//---

	// cameras
	enum CameraDirs {CAMERA_UP, CAMERA_DOWN} CameraDir; //+++
	void ToggleOnboardCamera(CameraDirs Dir);
	
	// PO, Astro and Radio
	char PoAstroName[255], PoRadioName[255];			//+++
	double PoAstroMass, PoRadioMass;					//---
	ATTACHMENTHANDLE hForPoAstro, hForPoRadio;			//---
	OBJHANDLE hPoAstro, hPoRadio;						//---
	void SetupPoAstro(), SetupPoRadio();
	void SeparatePoAstro(), SeparatePoRadio();

	// ALS assembly
//	enum als_types {LANDER, SATELLITE} AlsType;			//+++
	int AlsType;

	double AlsMass;										//---
	ATTACHMENTHANDLE hForAls, hForAls9, hForAls10;		//---
	OBJHANDLE hAls;										//---
	void SetupAls();
	void SeparateAls();
	void InflateAirbags();
	char AlsName[255];									//+++

	// animations
	bool bAnimate;										//???
	double Simdt;										//---
    enum anim_status { INITIAL, EXTENDED, BROKEN } AnimStatus;		//+++
	UINT ExtendAnimation, Brake1Animation, Brake2Animation;			//---
	double AnimationPercent;							//???
	void DefineAnimations(); 
	void Animate();
	void AnimationStart();
	void AnimationEnd();

	// focus controll
	int bFocusAll;										//+++
	int bSwitchFocus; // to ALS, after its separation	//+++

	// general configuration
	double MaxTimeAccel;		//+++
	void UpdatePhysics();
	void SetTimeAccelOff();
	void LimitTimeAccel();


		// AUTOPILOTS
	bool bAutopilot;			//+++ 
	OBJHANDLE hMoon;			//---
	OBJHANDLE hEarth;			//--- used only for Luna10
	double Alt;					//---
	double Timer;				//+++
	double AltTurn;				//+++ temp (until hyperbolic), used only for Luna10
	void ToggleAutopilot();
	void DisableAutopilot();
	void EnableAutopilot();
	bool CanReenableAutopilot();

	// MCC Autopilot
	enum mcc_states { MCC_NONE, MCC_ORIENT, MCC_STEADY, MCC_BURN } MccState;
	double YMoonDirLast;		// +--
	double ZVelDirLast;			// +--
	double PastStepPericenter;	// -++
	double InitialPericenter;	// -++
	bool MccIsNeeded();
	void MccAutopilotOrient();
	void MccAutopilotWait();
	void MccAutopilotBurn();

	// Landing Autopilot
	double AltBrake;			//+++, altitude to start braking at
	double Lap;					//+++, distance covered by autopilot velocity integration along the trajectory
	double Pmain;				//+--, main thrust controlled by autopilot
	double VApLand[100];		//-++
	double LApLand[100];		//-++
	char ProgramPoint[256];		//+++

	void CalcAltBrake();
	void LandingAutopilot();


		// Check-Do pairs for the timestep. 
		// Currently active 'Check' runs on every timestep and checks particular condition.
		// When condition is satisfied, the currently assigned 'Do' event is fired.
		// After performing its task, 'Do' function switches the function pointers 
		// to the next appropriate Check-Do pair.

	// enumenration and helper fonctions to get-set-save-restore Check-Do pair 
	// (when enumerating states, we actually look at Check, not at Do)
    enum chk_do_types 
	{ 
		CDT_DETACHED,	// are we still attached to parent object (block L)?
		CDT_MOONINFL,	// did we enter Moon influence spere?
		CDT_MCCALT,		// did we reach midcource correction altitude?
		CDT_MCCEND,		// are we done with midcourse correction maneuver?

		CDT_TIME1000,	// did we reach altitude with limited time acceleration?
		CDT_TIME100,	// did we reach altitude with limited time acceleration?
		
		CDT_TURN,		// did we reach the retrograde turn altitude? 
		CDT_IGNALT,		// did we reach ignition altitude? ONLY LUNA-9!
		CDT_IGNTIME,	// did we reach ignition time? ONLY LUNA-10!
		CDT_INFLEND,	// is airbags inflation complete?
		CDT_PARA,		// did we reach parachuting velocity? ONLY LUNA-9!
		CDT_PERI,		// did we reach the minimal perigee orbit? ONLY LUNA-10!
		CDT_TOUCHDOWN,	// did the probe end touch the ground? ONLY LUNA-9!
		CDT_MECO,		// did we reach MECO time? ONLY LUNA-10!
		CDT_INERT		// don't really check anything, always return false
	} 
	ChkDoType;	// current CheckDo type for saving-restoring

	// save-restore functions, work when saving-loading scenario or switching to and from manual mode
	void SaveCheckDo();
	void RestoreCheckDo();

	// function pointers for currently active Check-Do pair
	bool (Luna9KTDU::*pCheckCondition)();
	void (Luna9KTDU::*pDoAction)();

	// the actual check-Do pairs in the approximate sequencing order 
	// (the real sequence is conditional and may vary.

	bool CheckDetach();				// are we still attached to parent object (block L)?
	void DoDetach();				// detached from block L!

	bool CheckMoonInfluence();		// did we enter Moon influence spere?
	void DoEnterMoonInfluence();	// entered Moon influence spere!

	bool CheckMCCAlt();				// did we reach midcource correction altitude?
	void DoMCCAlt();				// start midcource correction maneuver!

	bool CheckMCCEnd();				// are we done with midcourse correction maneuver?
	void DoMCCEnd();				// midcourse correction maneuver is completed!

	bool CheckTimeAccel1000();		// did we reach altitude with limited time acceleration?
	void DoTimeAccel1000();			// update max time acceleration!

	bool CheckTimeAccel100();		// did we reach altitude with limited time acceleration?
	void DoTimeAccel100();			// update max time acceleration!

	bool CheckTurn();				// did we reach the retrograde turn altitude?
	void DoTurn();					// start retrograde turn!

	bool CheckIgnitionAlt();		// did we reach ignition altitude? ONLY LUNA-9!
	void DoIgnitionAlt();			// reached ignition altitude! ONLY LUNA-9!

	bool CheckIgnitionTime();		// did we reach ignition time? ONLY LUNA-10!
	void DoIgnitionTime();			// reached ignition time! ONLY LUNA-10!

	bool CheckInflationEnd();		// is airbags inflation complete?
	void DoInflationEnd();			// airbags inflation is complete!

	bool CheckParachuting();		// did we reach parachuting velocity? ONLY LUNA-9!
	bool CheckPericenter();			// did we reach the minimal perigee orbit? ONLY LUNA-10!
	void DoLowStage();				// switch engine to low stage

	bool CheckTouchdown();			// did the probe end touch the ground? ONLY LUNA-9!
	bool CheckMECO();				// did we reach MECO time? ONLY LUNA-10!
	void DoArrival();				// arrived at destination!

	bool CheckInert();				// don't really check anything, always return false


	// sound
	int SoundLibID;

	// helper functions
	void LocalRot(const VECTOR3 vecGlob, VECTOR3 &vecLoc);

	// output
	NOTEHANDLE MessagePort;			//---
	double msgStartTime;			//+--
	bool bMessageDisplayed;			//+--
	void ShowAnnotationMessage(char *Message);
	void ShowPermanentString(chk_do_types ChkDo, bool bIgnoreFocus = false);

	void DebugSomething();
};


