#include "Luna9PO.h"
#include "time.h"
#include "stdio.h"


Luna9PO::Luna9PO (OBJHANDLE hObj, int fmodel): VESSEL2M (hObj, fmodel)
{
	// class-level defaults
	PoType = POT_ASTRO;  // this should be DEFINITELY overridden, but for pure safety, assign something to it

	// attachment point
	CreateAttachment( TOPARENT, _V(0,0,0), _V(0,0,1), _V(1,0,0), "ForKTDU", NOTLOOSE );
}


// General initialization of general params, called before config file
void Luna9PO::clbkSetClassCaps (FILEHANDLE cfg)
{
	// Set geometry
	SetSize (0.5);

	// Set physics 
	SetEmptyMass (150);
	SetLiftCoeffFunc (0);

	SetPMI (_V(1.0, 1.0, .5));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetRotDrag (_V(0.5, 0.5, 0.5));
	SetCW (0.2, 0.2, 0.2, 0.2);

		// surface: set for the "jetisoned" case
	SetSurfaceFrictionCoeff (10.0, 10.0);
}	


void Luna9PO::ParseScenarioLineEx(char *line, void *status)
{
	char tempstr[255];

	// PO type: Astro or Radio? Default is Astro.
    if (!strnicmp (line, "PO_TYPE", 7)) 
	{
		sscanf (line+7, "%s", &tempstr);
		if (tempstr[0] == 'R')
			PoType = POT_RADIO; 
	}

	// VESSEL2M and VESSEL2 stuff
	else 
		VESSEL2M::ParseScenarioLineEx (line, status);
}


// Save status to scenario file
void Luna9PO::clbkSaveState (FILEHANDLE scn)
{
	// do inherited VESSEL2M stuff before writing custom stuff
	VESSEL2M::clbkSaveState (scn);

	// even though ASTRO is default, store it explicitly for better scn file readability
	if (PoType == POT_ASTRO)
		oapiWriteScenario_string (scn, "PO_TYPE", "ASTRO");
	else
		oapiWriteScenario_string (scn, "PO_TYPE", "RADIO");
}


// called after all vessel in scn are created, but before first timestep
void Luna9PO::FinishConfiguration()
{
	char MeshName[256];

	// override some VESSEL2 defaults for crash effects
	VvCrash = 30; 
	VhCrash = 30;
	VMaxBlast = 1000; 

	// 
	if (PoType == POT_ASTRO)
	{
		strcpy(MeshName, "Luna9PoAstro");
		QMax = DRand(60000, 5000);
		SetCrossSections (_V(0.5, 0.5, 0.5));
		SetTouchdownPoints (_V( 1, -0.4, 1), _V(-1, -0.2,-1), _V( 1, -0.0,-1));
	}
	else  // POT_RADIO
	{
		strcpy(MeshName, "Luna9PoRadio");
		QMax = DRand(50000, 5000);
		SetCrossSections (_V(0.55, 0.55, 0.55));
		SetTouchdownPoints (_V(-1, 0.3,-1), _V( 1, -0.2, 1), _V( 1, 0.1,-1));
	}

	// finish loading meshes
	VECTOR3 MeshOffset = _V(0, 0, 0);
	ClearMeshes();
	int MeshID = AddMesh (MeshName, &MeshOffset);
	SetMeshVisibilityMode (MeshID, MESHVIS_ALWAYS);

	// do VESSEL2M stuff
	VESSEL2M::FinishConfiguration();
}




void Luna9PO::clbkPostStep(double simt, double simdt, double mjd)
{
	// common processing for all VESSEL2-derived objects
	TimestepForVESSEL2M();
}





// ==============================================================
// API interface
// ==============================================================

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	srand((unsigned)time( NULL ));
	return new Luna9PO (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) 
		delete (Luna9PO*)vessel;
}



