
#include "Molniya1.h"
#include <stdio.h>
#include <math.h>
#include <time.h>


// default constructor
Molniya1::Molniya1 (OBJHANDLE hObj, int fmodel): VESSEL2 (hObj, fmodel)
{
}



// General initialization of general params, called before config file
void Molniya1::clbkSetClassCaps(FILEHANDLE cfg)
{
	// size and geometry
	SetSize (2.0);
	SetEmptyMass (1550.0);
	SetPMI (_V(1, 1, 1));
	SetCrossSections (_V(5, 5, 2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetRotDrag (_V(0.5, 0.5, 0.5));
	SetCW (0.2, 0.2, 0.2, 0.2);
	SetLiftCoeffFunc (0);
	SetSurfaceFrictionCoeff(10,10);
	SetCOG_elev (0);
	SetTouchdownPoints (_V(0, 4, 0), _V(-4, -4, 0), _V(4, -4, 0));

	// mesh
	VECTOR3 MeshOffset = _V(0, 0, 0);
	ClearMeshes();
	int MeshID = AddMesh ("Molniya1", &MeshOffset);
	SetMeshVisibilityMode (MeshID, MESHVIS_ALWAYS);

	// engines
	MainTank = CreatePropellantResource(100);
	MainEngine = CreateThruster(_V(0, 0, 2.5), _V(0,0,-1), 1960, MainTank, 2989, 2989);
	MainTH[0] = MainEngine;
	CreateThrusterGroup(MainTH,1,THGROUP_MAIN);

	PARTICLESTREAMSPEC ExhaustStream = { 0,			// flags, reserved
		0.10,			// srcsize, particle size at creation, m
		0.1,			// srcrate, average particle generation rate, Hz   
		20,			// V0, average particle emission velocity, m/s
		-0.05,			// srcspread, emission velocity distribution randomisation
		0.04,			// lifetime, average particle lifetime [s]
		7,			// growthrate, particle growth rate [m/s]
		0.1,			// atmslowdown, deceleration rate b in atmosphere, defined as v = v0 e- bdt 
		PARTICLESTREAMSPEC::EMISSIVE, PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5, PARTICLESTREAMSPEC::ATM_PLOG, 0, 0 };
	AddExhaustStream(MainTH[0], &ExhaustStream);

	SURFHANDLE ExhaustTexture = oapiRegisterExhaustTexture ("Molniya1\\exhaust_crcs");
	AddExhaust(MainTH[0], 1.5, 0.3, -0.1, ExhaustTexture);

	MakeRCS();

	// animations
	DefineAnimations();

	// Usually it is done in clbkLoadStateEx. But we don't have it, no need so far.
	// now that we know final configuration, finish configurng the new vessel
	AntennaState = STORED;
	AntennaPercent = 0;

	BoomsState = STORED;
	BoomsPercent = 0;

	ArmsState = STORED;
	ArmsPercent = 0;

	SolarState = STORED;
	SolarPercent = 0;

	// timers delaying unfolding from the scenario load or from separating from the booster.
	// Start solar panel animation just a little earlier, to try to distract from the initial amtenna animation glitch
	AntennaTimer = 3;
	SolarTimer = 2.9;
	bWasAttached = false;
}



void Molniya1::clbkPostStep(double simt, double simdt, double mjd)
{

	// Are we still attached to parent rocket? If yes, do nothing... at least until
	// any failure modes are introduced that may occur when stil lattached.
	if (Attached())
	{
		bWasAttached = true;
		return;
	}

	// if we just loaded scenario, deploy everything almost immediately.
	// If we are separating from the rocket, increase delay by 10 seconds
	if (bWasAttached)
	{
		// sound of pyros separating us from Block L
		if (AntennaTimer == 3)
			PlayVesselWave3(SoundLibID, SND_PYRO_A);

		bWasAttached = false;
		AntennaTimer += 10;
		SolarTimer += 10;
	}


	// everything is deployed? don't do anything else
	if ((AntennaState == DEPLOYED) && (ArmsState == DEPLOYED) && (SolarState == DEPLOYED))
		return;


	// check timers
	if (AntennaState == STORED)
		if ((AntennaTimer -= simdt) < 0)
		{
			AntennaState = DEPLOYING;
			PlayVesselWave3(SoundLibID, SND_PYRO_A);
		}

	if (SolarState == STORED)
		if ((SolarTimer -= simdt) < 0)
		{
			SolarState = DEPLOYING;
			PlayVesselWave3(SoundLibID, SND_PYRO_P);
		}

	// do the anomation...
	double dPercent;


	if (AntennaState == DEPLOYING) 
	{
		double AntennaSpeedOpen = 10.3;// + AntennaPercent*1.7; // ends at 2
		dPercent = simdt * AntennaSpeedOpen;

		if (AntennaPercent < 1.0)
			AntennaPercent = min (1.0, AntennaPercent + dPercent);
		else
		{
			// done!
			AntennaPercent = 1;
			AntennaState = DEPLOYED;
			PlayVesselWave3(SoundLibID, SND_CLANG_A);

			// now, start booms deployment
			BoomsState = DEPLOYING;
		}

		SetAnimation (AntennaAnimations[0], AntennaPercent);
		SetAnimation (AntennaAnimations[1], AntennaPercent);
	} 


	if (BoomsState == DEPLOYING) 
	{
		double BoomSpeedOpen = 0.3 + BoomsPercent*1.7; // ends at 2
		dPercent = simdt * BoomSpeedOpen;

		if (BoomsPercent < 1.0)
			BoomsPercent = min (1.0, BoomsPercent + dPercent);
		else
		{
			// done!
			BoomsPercent = 1;
			BoomsState = DEPLOYED;
			PlayVesselWave3(SoundLibID, SND_CLANG_B);

			// now, start arms deployment
			ArmsState = DEPLOYING;
		}

		SetAnimation (BoomAnimations[0], BoomsPercent);
		SetAnimation (BoomAnimations[1], BoomsPercent);
	} 
	

	if (ArmsState == DEPLOYING) 
	{
		double ArmSpeedOpen = 1.5 + BoomsPercent;// ends at 2.5
		dPercent = simdt * ArmSpeedOpen;

		if (ArmsPercent < 1.0)
			ArmsPercent = min (1.0, ArmsPercent + dPercent);
		else
		{
			// done!
			ArmsPercent = 1;
			ArmsState = DEPLOYED;
			PlayVesselWave3(SoundLibID, SND_CLANG_B);

			// now, start arms deployment
//			AntennaState = DEPLOYING;
		}

		SetAnimation (ArmAnimations[0], ArmsPercent);
		SetAnimation (ArmAnimations[1], ArmsPercent);
	} 



	if (SolarState == DEPLOYING) 
	{
		double SolarSpeedOpen = 0.1 + SolarPercent*1.9;// ends at 2
		dPercent = simdt * SolarSpeedOpen;

		if (SolarPercent < 1.0)
			SolarPercent = min (1.0, SolarPercent + dPercent);
		else
		{
			// done!
			SolarPercent = 1;
			SolarState = DEPLOYED;
			PlayVesselWave3(SoundLibID, SND_CLANG_P);
		}

		for (int i = 0; i < 6; i++)
		{
			SetAnimation (SpInnerAnimations[i], SolarPercent);
			SetAnimation (SpOuterAnimations[i], SolarPercent);
		}
	} 
}


void Molniya1::DefineAnimations()
{
/*
	IMPORTANT NOTE!
	We will have to start from animating parabolic antennas before deploying the booms,
	because scaling animations (used for antenna) are limited to the native axes of the vessel,
	and cannot be done later, when booms and arms are deployes, and antenna axes rotated!
	So, we have to do antennas first, even if it "embeds" the antenna mesh into the satellite body 
	for a short moment.
*/
	
	// antennas

	MGROUP_SCALE *Antennas[2];
	static UINT grAntenna1[2] = {220, 221}; 
	static UINT grAntenna2[2] = {222, 223}; 

	Antennas[0] = new MGROUP_SCALE(0, grAntenna1, 2, _V( 0.957,0.0,0.538), _V(3, 3, 0.3));
	Antennas[1] = new MGROUP_SCALE(0, grAntenna2, 2, _V(-0.957,0.0,0.538), _V(3, 3, 0.3));

	AntennaAnimations[0] = CreateAnimation(0);
	AddAnimationComponent(AntennaAnimations[0], 0, 1, Antennas[0]);

	AntennaAnimations[1] = CreateAnimation(0);
	AddAnimationComponent(AntennaAnimations[1], 0, 1, Antennas[1]);

	
	// antenna booms

	MGROUP_ROTATE *Booms[2];

	static UINT grBoom0[4] = {219,218,220,221}; 
	static UINT grBoom1[4] = {181,180,222,223}; 

	Booms[0] = new MGROUP_ROTATE(0, grBoom0, 4, _V( 0.957,0.0,-0.678), _V(0, 1, 0), (float)( 90*RAD));
	Booms[1] = new MGROUP_ROTATE(0, grBoom1, 4, _V(-0.957,0.0,-0.678), _V(0, 1, 0), (float)(-90*RAD));

	BoomAnimations[0] = CreateAnimation(0);
	AddAnimationComponent(BoomAnimations[0], 0, 1, Booms[0]);

	BoomAnimations[1] = CreateAnimation(0);
	AddAnimationComponent(BoomAnimations[1], 0, 1, Booms[1]);


	// antenna arms

	MGROUP_ROTATE *Arms[2];

	static UINT grArm0[3] = {218,220,221}; 
	static UINT grArm1[3] = {180,222,223}; 

	Arms[0] = new MGROUP_ROTATE(0, grArm0, 3, _V( 2.043,0.0,-0.678), _V(0, 1, 0), (float)( 60*RAD));
	Arms[1] = new MGROUP_ROTATE(0, grArm1, 3, _V(-2.043,0.0,-0.678), _V(0, 1, 0), (float)(-60*RAD));

	ArmAnimations[0] = CreateAnimation(0);
	AddAnimationComponent(ArmAnimations[0], 0, 1, Arms[0]);

	ArmAnimations[1] = CreateAnimation(0);
	AddAnimationComponent(ArmAnimations[1], 0, 1, Arms[1]);


	// solar panels, inner and outer

	MGROUP_ROTATE *SpInner[6];
	void  *SpInnerHandles[6];
	static UINT grInner[6][3] = {183,184,182,   189,188,190,   215,216,217,   207,206,208,   194,195,196,   204,205,203}; 
	static double InnerRefPoints[6][3] = {0.0,-0.998,-0.992,   0.0,0.998,-0.992,   -0.866,0.493,-0.992,   0.862,-0.506,-0.992,   0.866,0.5,-0.992,   -0.861,-0.503,-0.992};
	static double Axes[6][3] = {1,0,0,   1,0,0,   0.37,0.63,0,   -0.37,-0.63,0,   0.37,-0.63,0,   -0.37,0.63,0};
	static double InnerAngles[6] = {95, -95, -130, -130, -130, -130};

	MGROUP_ROTATE *SpOuter[6];
	static UINT grOuter[6][3] = {185,186,187,   191,192,193,   212,213,214,   209,210,211,   197,198,199,   200,201,202};
	static double OuterRefPoints[6][3] = {0.0,-0.869,0.543,   0.0,0.869,0.543,   -0.751,0.43,0.543,   0.751,-0.437,0.543,   0.753,0.434,0.543,   -0.747,-0.434,0.543};
	static double OuterAngles[6] = {-180, 180, 180, 180, 180, 180};


	for (int i = 0; i < 6; i++)
	{
		VECTOR3 InnerRefPoint = _V(InnerRefPoints[i][0], InnerRefPoints[i][1], InnerRefPoints[i][2]);
		VECTOR3 Axis = _V(Axes[i][0],  Axes[i][1], Axes[i][2]);
		SpInner[i] = new MGROUP_ROTATE(0, grInner[i], 3, InnerRefPoint, Axis, (float)(InnerAngles[i]*RAD));
		SpInnerAnimations[i] = CreateAnimation(0);
		SpInnerHandles[i] = AddAnimationComponent(SpInnerAnimations[i], 0, 1, SpInner[i]);

		VECTOR3 OuterRefPoint = _V(OuterRefPoints[i][0], OuterRefPoints[i][1], OuterRefPoints[i][2]);
		SpOuter[i] = new MGROUP_ROTATE(0, grOuter[i], 3, OuterRefPoint, Axis, (float)(OuterAngles[i]*RAD));
		SpOuterAnimations[i] = CreateAnimation(0);
		AddAnimationComponent(SpOuterAnimations[i], 0, 1, SpOuter[i], SpInnerHandles[i]);
	}


}


	// simpliest RCS, symmetrical forces
void Molniya1::MakeRCS()
{
	const double RCSThrust = 5;
	const double RCSISP = 2989;
	THRUSTER_HANDLE thRot[2];


	thRot[0] = CreateThruster (_V(-6, 0, 0), _V(0,  1,0), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V( 6, 0, 0), _V(0, -1,0), RCSThrust, MainTank, RCSISP);
	RightRoll = CreateThrusterGroup (thRot, 2, THGROUP_ATT_BANKRIGHT);

	thRot[0] = CreateThruster (_V( 6, 0, 0), _V(0,  1,0), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V(-6, 0, 0), _V(0, -1,0), RCSThrust, MainTank, RCSISP);
	LeftRoll = CreateThrusterGroup (thRot, 2, THGROUP_ATT_BANKLEFT);

	thRot[0] = CreateThruster (_V( 6,0,0), _V(0, 0,-1), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V(-6,0,0), _V(0, 0, 1), RCSThrust, MainTank, RCSISP);
	RightYaw = CreateThrusterGroup (thRot, 2, THGROUP_ATT_YAWRIGHT);

	thRot[0] = CreateThruster (_V(-6,0,0), _V(0, 0,-1), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V( 6,0,0), _V(0, 0, 1), RCSThrust, MainTank, RCSISP);
	LeftYaw = CreateThrusterGroup (thRot, 2, THGROUP_ATT_YAWLEFT);

	thRot[0] = CreateThruster (_V(0,0, 6), _V(0,  1, 0), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V(0,0,-6), _V(0, -1, 0), RCSThrust, MainTank, RCSISP);
	PitchUp = CreateThrusterGroup (thRot, 2, THGROUP_ATT_PITCHUP);

	thRot[0] = CreateThruster (_V(0,0, 6), _V(0, -1, 0), RCSThrust, MainTank, RCSISP);
	thRot[1] = CreateThruster (_V(0,0,-6), _V(0,  1, 0), RCSThrust, MainTank, RCSISP);
	PitchDown = CreateThrusterGroup (thRot, 2, THGROUP_ATT_PITCHDOWN);
}



// Are we still attached to parent rocket? 
bool Molniya1::Attached()
{
	DWORD ParentAttCount = AttachmentCount(true);  // true means "to parent"
	if (ParentAttCount > 0)
	{		
		// still attached?
		OBJHANDLE hParent = GetAttachmentStatus(GetAttachmentHandle(true, 0));
		if (hParent != NULL)
			return true;
	}

	return false;
}

// Enforce camera dir switch here:
bool Molniya1::clbkLoadGenericCockpit ()
{
	oapiSetDefRCSDisplay(0);

	SetCameraOffset(_V(0.8, -0.6, 1)); 
	SetCameraDefaultDirection(_V(0,0,-1.3));

	return true;
}


		// SOUND
void Molniya1::clbkPostCreation (void)
{
	SoundLibID = ConnectToOrbiterSoundDLL3(GetHandle());

	SoundOptionOnOff3(SoundLibID, PLAYMAINTHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYATTITUDETHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYLANDINGANDGROUNDSOUND, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAIRSPEED, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAMBIANCEWHENLANDED, true);

	SoundOptionOnOff3(SoundLibID, PLAYCOUNTDOWNWHENTAKEOFF, false);
	SoundOptionOnOff3(SoundLibID, PLAYWHENATTITUDEMODECHANGE, false);
	SoundOptionOnOff3(SoundLibID, PLAYGPWS, false);
	SoundOptionOnOff3(SoundLibID, PLAYHOVERTHRUST, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKINGSOUND, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADARBIP, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKLANDCLEARANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINAIRCONDITIONING, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINRANDOMAMBIANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADIOATC, false);
	SoundOptionOnOff3(SoundLibID, DISPLAYTIMER, false);

	RequestLoadVesselWave3(SoundLibID, SND_PYRO_A, "Sound\\Molniya1\\Pyro.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_PYRO_P, "Sound\\Molniya1\\Pyro.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_CLANG_A, "Sound\\Molniya1\\Clang.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_CLANG_P, "Sound\\Molniya1\\Clang.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_CLANG_B, "Sound\\Molniya1\\Clang.wav", BOTHVIEW_FADED_FAR);

	ReplaceStockSound3(SoundLibID, "Sound\\Molniya1\\Motor.wav", REPLACE_RCS_THRUST_SUSTAIN);
	ReplaceStockSound3(SoundLibID, "Sound\\Molniya1\\Scilence.wav", REPLACE_RCS_THRUST_ATTACK);
}



void Molniya1::DebugSomething()
{
//	sprintf(oapiDebugString(), "test: %d", BoomsPercent);
}



// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	// seed random generator
	srand((unsigned)time( NULL ));

	return new Molniya1 (hvessel, flightmodel);
}


// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) 
		delete (Molniya1*)vessel;
}

